// #include <GLFW/glfw3.h>
// #include <imgui.h>
// #include <imgui_impl_glfw.h>
// #include <imgui_impl_opengl3.h>
// #include <stdio.h>

// int main(void) {
//     GLFWwindow* window;
//     if (!glfwInit()) {
//         printf("Couldn't initialize GLFW!\n");
//         return 1;
//     }

//     window = glfwCreateWindow(640, 480, "Sentinal", NULL, NULL);
//     if (!window) {
//         printf("Couldn't create window\n");
//         return 1;
//     }

//     glfwMakeContextCurrent(window);

//     // Initialize OpenGL loader (you might need to include and initialize GLEW or glad here)

//     // Initialize ImGui
//     IMGUI_CHECKVERSION();
//     ImGui::CreateContext();
//     ImGuiIO& io = ImGui::GetIO(); (void)io;
//     ImGui::StyleColorsDark();

//     // Initialize ImGui backends
//     ImGui_ImplGlfw_InitForOpenGL(window, true);
//     ImGui_ImplOpenGL3_Init("#version 130");

//     while (!glfwWindowShouldClose(window)) {
//         glfwPollEvents();

//         // Start ImGui frame
//         ImGui_ImplOpenGL3_NewFrame();
//         ImGui_ImplGlfw_NewFrame();
//         ImGui::NewFrame();

//         // Your ImGui code here
//         ImGui::Begin("Hello, ImGui!");
//         ImGui::Text("This is a simple window.");
//         ImGui::End();

//         // Render
//         ImGui::Render();
//         int display_w, display_h;
//         glfwGetFramebufferSize(window, &display_w, &display_h);
//         glViewport(0, 0, display_w, display_h);
//         glClearColor(0.45f, 0.55f, 0.60f, 1.00f);
//         glClear(GL_COLOR_BUFFER_BIT);
//         ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());

//         glfwSwapBuffers(window);
//     }

//     // Cleanup
//     ImGui_ImplOpenGL3_Shutdown();
//     ImGui_ImplGlfw_Shutdown();
//     ImGui::DestroyContext();

//     glfwDestroyWindow(window);
//     glfwTerminate();

//     return 0;
// }

extern "C" {
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/avutil.h>
}

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <imgui.h>
#include <imgui_impl_glfw.h>
#include <imgui_impl_opengl3.h>
#include <stdio.h>

// Function to create a texture from an OpenCV frame
GLuint createTextureFromFrame(const cv::Mat& frame) {
    GLuint textureID;
    glGenTextures(1, &textureID);
    glBindTexture(GL_TEXTURE_2D, textureID);

    // Set texture parameters
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    // Upload the frame to the texture
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, frame.cols, frame.rows, 0, GL_BGR, GL_UNSIGNED_BYTE, frame.data);

    glBindTexture(GL_TEXTURE_2D, 0);
    return textureID;
}

int main(void) {
    // Initialize GLFW
    if (!glfwInit()) {
        printf("Couldn't initialize GLFW!\n");
        return 1;
    }

    // Create a windowed mode window and its OpenGL context
    GLFWwindow* window = glfwCreateWindow(640, 480, "Sentinal", NULL, NULL);
    if (!window) {
        printf("Couldn't create window\n");
        glfwTerminate();
        return 1;
    }

    // Make the window's context current
    glfwMakeContextCurrent(window);

    // Initialize GLEW
    if (glewInit() != GLEW_OK) {
        printf("Couldn't initialize GLEW!\n");
        return 1;
    }

    // Initialize ImGui
    IMGUI_CHECKVERSION();
    ImGui::CreateContext();
    ImGuiIO& io = ImGui::GetIO(); (void)io;
    ImGui::StyleColorsDark();

    // Initialize ImGui backends
    ImGui_ImplGlfw_InitForOpenGL(window, true);
    ImGui_ImplOpenGL3_Init("#version 130");

    // OpenCV video capture
    cv::VideoCapture cap(0); // Change the argument to a video file path if needed
    if (!cap.isOpened()) {
        printf("Couldn't open video capture\n");
        return 1;
    }

    GLuint videoTexture = 0;

    // Main loop
    while (!glfwWindowShouldClose(window)) {
        glfwPollEvents();

        // Capture frame
        cv::Mat frame;
        cap >> frame; // Get the next frame from the video

        if (!frame.empty()) {
            // Create or update texture
            if (videoTexture == 0) {
                videoTexture = createTextureFromFrame(frame);
            } else {
                glBindTexture(GL_TEXTURE_2D, videoTexture);
                glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, frame.cols, frame.rows, GL_BGR, GL_UNSIGNED_BYTE, frame.data);
                glBindTexture(GL_TEXTURE_2D, 0);
            }
        }

        // Start ImGui frame
        ImGui_ImplOpenGL3_NewFrame();
        ImGui_ImplGlfw_NewFrame();
        ImGui::NewFrame();

        // ImGui window with video
        ImGui::Begin("Video Stream");
        if (videoTexture != 0) {
            ImGui::Image((void*)(intptr_t)videoTexture, ImVec2((float)frame.cols, (float)frame.rows));
        }
        ImGui::End();

        // Render ImGui
        ImGui::Render();
        int display_w, display_h;
        glfwGetFramebufferSize(window, &display_w, &display_h);
        glViewport(0, 0, display_w, display_h);
        glClearColor(0.45f, 0.55f, 0.60f, 1.00f);
        glClear(GL_COLOR_BUFFER_BIT);
        ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());

        // Swap buffers
        glfwSwapBuffers(window);
    }

    // Cleanup
    glDeleteTextures(1, &videoTexture);
    ImGui_ImplOpenGL3_Shutdown();
    ImGui_ImplGlfw_Shutdown();
    ImGui::DestroyContext();
    glfwDestroyWindow(window);
    glfwTerminate();

    return 0;
}